==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/fir/backend/jvm/FirJvmBackendExtension.kt <==
/*
 * Copyright 2010-2020 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.backend.jvm

import ksp.org.jetbrains.kotlin.backend.jvm.JvmBackendContext
import ksp.org.jetbrains.kotlin.backend.jvm.JvmBackendExtension
import ksp.org.jetbrains.kotlin.backend.jvm.ModuleMetadataSerializer
import ksp.org.jetbrains.kotlin.backend.jvm.metadata.MetadataSerializer
import ksp.org.jetbrains.kotlin.codegen.serialization.JvmSerializationBindings
import ksp.org.jetbrains.kotlin.fir.backend.Fir2IrComponents
import ksp.org.jetbrains.kotlin.fir.backend.FirMetadataSource
import ksp.org.jetbrains.kotlin.fir.declarations.FirClass
import ksp.org.jetbrains.kotlin.fir.declarations.FirDeclaration
import ksp.org.jetbrains.kotlin.fir.serialization.FirElementAwareStringTable
import ksp.org.jetbrains.kotlin.fir.serialization.FirElementSerializer
import ksp.org.jetbrains.kotlin.fir.serialization.TypeApproximatorForMetadataSerializer
import ksp.org.jetbrains.kotlin.fir.serialization.serializeAnnotations
import ksp.org.jetbrains.kotlin.ir.declarations.IrClass
import ksp.org.jetbrains.kotlin.ir.declarations.MetadataSource
import ksp.org.jetbrains.kotlin.metadata.ProtoBuf
import ksp.org.jetbrains.kotlin.metadata.serialization.MutableVersionRequirementTable
import ksp.org.jetbrains.kotlin.serialization.StringTableImpl
import ksp.org.jetbrains.kotlin.serialization.deserialization.builtins.BuiltInSerializerProtocol
import ksp.org.jetbrains.org.objectweb.asm.Type

class FirJvmBackendExtension(
    private val components: Fir2IrComponents,
    private val actualizedExpectDeclarations: Set<FirDeclaration>?
) : JvmBackendExtension {
    override fun createSerializer(
        context: JvmBackendContext,
        klass: IrClass,
        type: Type,
        bindings: JvmSerializationBindings,
        parentSerializer: MetadataSerializer?
    ): MetadataSerializer {
        return makeFirMetadataSerializerForIrClass(
            components.session,
            context,
            klass,
            bindings,
            components,
            parentSerializer,
            actualizedExpectDeclarations
        )
    }

    override fun createModuleMetadataSerializer(context: JvmBackendContext): ModuleMetadataSerializer = object : ModuleMetadataSerializer {
        override fun serializeOptionalAnnotationClass(metadata: MetadataSource.Class, stringTable: StringTableImpl): ProtoBuf.Class {

            require(metadata is FirMetadataSource.Class) { "Metadata is expected to be ${FirMetadataSource.Class::class.simpleName}" }

            val session = components.session
            val fir = metadata.fir

            val typeApproximator = TypeApproximatorForMetadataSerializer(session)
            // Get rid of special serializer extension after KT-57919, i.e. when we serialize all the annotations by default
            val firSerializerExtension = object : FirJvmSerializerExtension(
                session,
                JvmSerializationBindings(),
                context.state,
                // annotation can't have local delegated properties, it is safe to pass empty list
                localDelegatedProperties = emptyList(),
                typeApproximator,
                components,
                object : FirElementAwareStringTable {
                    override fun getQualifiedClassNameIndex(className: String, isLocal: Boolean): Int =
                        stringTable.getQualifiedClassNameIndex(className, isLocal)

                    override fun getStringIndex(string: String): Int = stringTable.getStringIndex(string)

                    override fun getPackageFqNameIndexByString(fqName: String): Int = stringTable.getPackageFqNameIndexByString(fqName)
                }
            ) {
                override fun serializeClass(
                    klass: FirClass,
                    proto: ProtoBuf.Class.Builder,
                    versionRequirementTable: MutableVersionRequirementTable,
                    childSerializer: FirElementSerializer,
                ) {
                    klass.serializeAnnotations(
                        session,
                        additionalMetadataProvider,
                        annotationSerializer,
                        proto,
                        BuiltInSerializerProtocol.classAnnotation
                    )
                    super.serializeClass(klass, proto, versionRequirementTable, childSerializer)
                }
            }
            val serializer = FirElementSerializer.create(
                session,
                components.scopeSession,
                fir,
                firSerializerExtension,
                parentSerializer = null,
                typeApproximator,
                context.config.languageVersionSettings
            )
            return serializer.classProto(fir).build()
        }
    }
}

==> symbol-processing-aa-embeddable-2.2.0-2.0.2-sources.jar: org/jetbrains/kotlin/fir/backend/jvm/FirJvmBackendExtension.kt <==
/*
 * Copyright 2010-2020 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.backend.jvm

import ksp.org.jetbrains.kotlin.backend.jvm.JvmBackendContext
import ksp.org.jetbrains.kotlin.backend.jvm.JvmBackendExtension
import ksp.org.jetbrains.kotlin.backend.jvm.ModuleMetadataSerializer
import ksp.org.jetbrains.kotlin.backend.jvm.metadata.BuiltinsSerializer
import ksp.org.jetbrains.kotlin.backend.jvm.metadata.MetadataSerializer
import ksp.org.jetbrains.kotlin.codegen.serialization.JvmSerializationBindings
import ksp.org.jetbrains.kotlin.fir.backend.Fir2IrComponents
import ksp.org.jetbrains.kotlin.fir.backend.FirMetadataSource
import ksp.org.jetbrains.kotlin.fir.declarations.FirClass
import ksp.org.jetbrains.kotlin.fir.declarations.FirDeclaration
import ksp.org.jetbrains.kotlin.fir.resolve.providers.firProvider
import ksp.org.jetbrains.kotlin.fir.serialization.FirElementAwareStringTable
import ksp.org.jetbrains.kotlin.fir.serialization.FirElementSerializer
import ksp.org.jetbrains.kotlin.fir.serialization.TypeApproximatorForMetadataSerializer
import ksp.org.jetbrains.kotlin.fir.serialization.serializeAnnotations
import ksp.org.jetbrains.kotlin.ir.declarations.IrClass
import ksp.org.jetbrains.kotlin.ir.declarations.MetadataSource
import ksp.org.jetbrains.kotlin.metadata.ProtoBuf
import ksp.org.jetbrains.kotlin.metadata.serialization.MutableVersionRequirementTable
import ksp.org.jetbrains.kotlin.serialization.StringTableImpl
import ksp.org.jetbrains.kotlin.serialization.deserialization.builtins.BuiltInSerializerProtocol
import ksp.org.jetbrains.org.objectweb.asm.Type

class FirJvmBackendExtension(
    private val components: Fir2IrComponents,
    private val actualizedExpectDeclarations: Set<FirDeclaration>?
) : JvmBackendExtension {
    override fun createSerializer(
        context: JvmBackendContext,
        klass: IrClass,
        type: Type,
        bindings: JvmSerializationBindings,
        parentSerializer: MetadataSerializer?
    ): MetadataSerializer {
        return makeFirMetadataSerializerForIrClass(
            components.session,
            context,
            klass,
            bindings,
            components,
            parentSerializer,
            actualizedExpectDeclarations
        )
    }

    override fun createModuleMetadataSerializer(context: JvmBackendContext): ModuleMetadataSerializer = object : ModuleMetadataSerializer {
        override fun serializeOptionalAnnotationClass(metadata: MetadataSource.Class, stringTable: StringTableImpl): ProtoBuf.Class {

            require(metadata is FirMetadataSource.Class) { "Metadata is expected to be ${FirMetadataSource.Class::class.simpleName}" }

            val session = components.session
            val fir = metadata.fir

            val typeApproximator = TypeApproximatorForMetadataSerializer(session)
            val firSerializerExtension = object : FirJvmSerializerExtension(
                session,
                JvmSerializationBindings(),
                context.state,
                // annotation can't have local delegated properties, it is safe to pass empty list
                localDelegatedProperties = emptyList(),
                typeApproximator,
                components,
                object : FirElementAwareStringTable {
                    override fun getQualifiedClassNameIndex(className: String, isLocal: Boolean): Int =
                        stringTable.getQualifiedClassNameIndex(className, isLocal)

                    override fun getStringIndex(string: String): Int = stringTable.getStringIndex(string)

                    override fun getPackageFqNameIndexByString(fqName: String): Int = stringTable.getPackageFqNameIndexByString(fqName)
                }
            ) {
                override val isOptionalAnnotationClassSerialization: Boolean
                    get() = true

                override fun serializeClass(
                    klass: FirClass,
                    proto: ProtoBuf.Class.Builder,
                    versionRequirementTable: MutableVersionRequirementTable,
                    childSerializer: FirElementSerializer,
                ) {
                    // Before 2.2, annotations were written to the `BuiltInsProtoBuf.classAnnotation` extension.
                    // Starting from 2.2, they are written to the `ProtoBuf.Class.annotation` field (KT-57919).
                    if (!context.config.metadataVersion.isAtLeast(2, 2, 0)) {
                        klass.serializeAnnotations(
                            session,
                            additionalMetadataProvider,
                            annotationSerializer,
                            proto,
                            BuiltInSerializerProtocol.classAnnotation
                        )
                    }
                    super.serializeClass(klass, proto, versionRequirementTable, childSerializer)
                }
            }
            val serializer = FirElementSerializer.create(
                session,
                components.scopeSession,
                fir,
                firSerializerExtension,
                parentSerializer = null,
                typeApproximator,
                context.config.languageVersionSettings
            )
            val file = session.firProvider.getFirClassifierContainerFileIfAny(fir.symbol)
            return serializer.classProto(fir, file).build()
        }
    }

    override fun createBuiltinsSerializer(): BuiltinsSerializer = FirBuiltInsSerializer(components.session, components.scopeSession)
}
